/* utils.c
 *
 *  Utility functions for xpsk31
 */

/*
 *  xpsk31: An application to transmit and receive
 *  Feld Hell signals using a computer's sound card
 *
 *  Copyright (C) 2003  Neoklis Kyriazis
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include "utils.h"
#include "support.h"

/* Signal samples buffer */
extern unsigned char *dsp_buff;

/* xpsk31 main window */
extern GtkWidget *main_window;

/* Receive pix buff */
extern GdkPixbuf *gbl_pixbuf;

/* QSO data record */
extern qso_record_t qso_record;

/* Runtime config data */
extern rc_data_t rc_data;

extern char
  label[9][17], /* Labels for F1-F9 commands */
  *macro[9];    /* Macros attached to F1-F9  */

/* Index to macros */
extern int gbl_macro_idx;

/*------------------------------------------------------------------------*/

/*  Save_QSO_Record()
 *
 *  Saves QSO record to .adif and .txt files
 */

  gboolean
Save_QSO_Record( void )
{
  /* For opening files */
  char file_name[64];

  /* Buffer for reading and printing to files */
  char buff[REC_BUFF_SIZE];
  char *buff_idx;

  int
	num_records, /* Number of QSO records    */
	num_logged,  /* Number of QSL's printed  */
	fopen_ret;   /* Open_File() return value */

  /* Create a file for QSO ADIF log if not open */
  strncpy( file_name, getenv("HOME"), 25 );
  strcat( file_name, "/xpsk31/log.adif" );
  fopen_ret = Open_File( &rc_data.log_adif_fp, file_name, "a" );
  if( fopen_ret == FILE_NEW )
  {
	snprintf( buff, REC_BUFF_SIZE,
		"    QSO Log file created in ADIF v1.0 format"
		" by %s %s <EOH>\n\n", PACKAGE, VERSION );
	if( ! File_Print(&rc_data.log_adif_fp, buff) )
	  return( FALSE );
  }
  else
	if( fopen_ret == -1 )
	  return( FALSE );

  /* Print QSO record to adif format file */
  snprintf( buff, REC_BUFF_SIZE,
	  "<CALL:%d>%s<QSO_DATE:8>%s<TIME_ON:4>%s\n"
	  "<FREQ:%d>%s<MODE:4>Hell<RST_SENT:3>%3s<EOR>\n\n",
	  (int)strlen(qso_record.dx_call), qso_record.dx_call,
	  qso_record.date_adif, qso_record.time_adif,
	  (int)strlen(qso_record.freq), qso_record.freq,
	  qso_record.dx_rst );
  if( ! File_Print(&rc_data.log_adif_fp, buff) )
	return( FALSE );

  if( fflush(rc_data.log_adif_fp) != 0 )
  {
	perror( file_name );
	ClearFlag( SAVE_RECORD );
	Error_Dialog(
		"Failed to fflush log.adif file\n"
		"Quit xpsk31 and correct", TRUE );
	return( FALSE );
  }

  /* Create a file for station log if not already open */
  strncpy( file_name, getenv("HOME"), 25 );
  strcat( file_name, "/xpsk31/log.txt" );
  fopen_ret = Open_File( &rc_data.station_log_fp, file_name, "r+" );
  if( fopen_ret == FILE_NEW )
  {
	/* Print a header to log file */
	snprintf( buff, REC_BUFF_SIZE,
		"|------------------------------------"
		"--------------------------------------|\n"
		"|          Station Log File - Created in"
		" Text format by %-9s         |\n"
		"|------------------------------------"
		"--------------------------------------|\n",
		PACKAGE"-"VERSION );
	if( ! File_Print(&rc_data.station_log_fp, buff) )
	  return( FALSE );

	/* Print an initial (0) count of records and qsl's */
	snprintf( buff, REC_BUFF_SIZE,
		"|  Num of QSO Records:%-6d  Records Logged:%-6d"
		"  Size of Records:%-4d  |\n"
		"|------------------------------------"
		"--------------------------------------|\n",
		0, 0, RECORD_SIZE );
	if( ! File_Print(&rc_data.station_log_fp, buff) )
	  return( FALSE );
  }
  else
	if( fopen_ret == -1 )
	  return( FALSE );

  /*** Increment and print Record and QSL count ***/
  /* Go to beginning of log file */
  fseek( rc_data.station_log_fp, 0, SEEK_SET );

  /* Find line with "Number of QSO Records:" */
  if( ! Locate_Line(&buff_idx, buff, "QSO Records:", rc_data.station_log_fp) )
	return( FALSE );
  num_records = atoi(buff_idx);

  /* Find "Number of QSO's QSL'd:" string */
  if( ! Locate_String(&buff_idx, buff, "Records Logged:") )
	return( FALSE );
  num_logged = atoi(buff_idx);

  /* Go back to beginning of line and print new record */
  fseek( rc_data.station_log_fp, -(1 + strlen(buff)), SEEK_CUR );
  snprintf( buff, REC_BUFF_SIZE,
	  "|  Num of QSO Records:%-6d  Records Logged:%-6d"
	  "  Size of Records:%-4d  |\n"
	  "|------------------------------------"
	  "--------------------------------------|\n",
	  ++num_records, num_logged, RECORD_SIZE );
  if( ! File_Print(&rc_data.station_log_fp, buff) )
	return( FALSE );

  /*** Print record in text log file ***/
  fseek( rc_data.station_log_fp, 0, SEEK_END );
  snprintf( buff, REC_BUFF_SIZE,
	  "|DX/ CALL: %-14s NAME: %-12s QTH: %-12s  LOC: %-6s|\n"
	  "|MY/ CALL: %-14s ZONE: %-12s QTH: %-12s  LOC: %-6s|\n"
	  "|QSO DATE: %-11s    TIME: %-5s UTC   FREQ: %-13s QSL: NO    |\n"
	  "|QSO MODE: %-11s DX-REPT: %-3s      MY-REPT: %-3s     VIA: %-12s|\n"
	  "|MY TRANS: %-11s   POWER: %-9s    ANT: %-15s          |\n"
	  "|MY RECVR: %-11s   N.FIG: %-7s      ANT: %-15s          |\n"
	  "| REMARKS: %-45s                   |\n"
	  "|------------------------------------"
	  "--------------------------------------|\n",
	  qso_record.dx_call, qso_record.dx_name, qso_record.dx_qth, qso_record.dx_loc,
	  rc_data.call, rc_data.zone, rc_data.qth, rc_data.loc,
	  qso_record.date, qso_record.time, qso_record.freq,
	  qso_record.mode, qso_record.dx_rst, qso_record.my_rst, qso_record.via,
	  rc_data.tx, rc_data.tx_power, rc_data.tx_ant,
	  rc_data.rx, rc_data.rx_nfig, rc_data.rx_ant, qso_record.remarks );
  if( ! File_Print(&rc_data.station_log_fp, buff) )
	return( FALSE );

  fflush( rc_data.station_log_fp );
  fflush( rc_data.log_adif_fp );

  ClearFlag( SAVE_RECORD );

  return( TRUE );

} /* Save_QSO_Record() */

/*------------------------------------------------------------------------*/

/*  Process_RST()
 *
 *  Processes RST-out and RST-in fields
 */

  void
Process_RST( GtkEditable *editable )
{
  G_CONST_RETURN gchar *rst;
  int len, idx;

  /* Validate RST: numerics only, */
  /* min 337 max 559, ignore blank field */
  rst = gtk_entry_get_text( GTK_ENTRY(editable) );
  len = strlen(rst);

  /* Ignore blank fields */
  if( len == 0 )
	return;

  for( idx = 0; idx < len; idx++ )
  {
	/* Reject non-numbers */
	if( (rst[idx] < '0') ||
		(rst[idx] > '9') )
	{
	  Bad_Entry_Dialog(
		  "Invalid character entered\n"
		  "Numbers only allowed" );
	  return;
	}

	/* Reject RST < 337 or > 599 */
	switch( idx )
	{
	  case 0:
		if( (rst[idx] < '3') ||
			(rst[idx] > '5') )
		{
		  Bad_Entry_Dialog(
			  "Number out of range\n"
			  "Min = 3 and Max = 5" );
		  return;
		}
		break;

	  case 1:
		if( rst[idx] < '3' )
		{
		  Bad_Entry_Dialog(
			  "Number out of range\n"
			  "Min = 3 and Max = 9" );
		  return;
		}
		break;

	  case 2:
		if( rst[idx] < '7' )
		{
		  Bad_Entry_Dialog(
			  "Number out of range\n"
			  "Min = 7 and Max = 9" );
		  return;
		}
	} /* switch( idx ) */

  } /* for( idx = 0; idx < len; idx++ ) */

} /* Process_RST() */

/*------------------------------------------------------------------------*/

/*  Select_Macro()
 *
 *  Selects a macro to be transmitted
 */

  void
Select_Macro( macro_num )
{
  /* Transmit a macro */
  if( isFlagClear(TRANSMIT_MACRO) )
  {
	SetFlag( TRANSMIT_MACRO );
	ClearFlag( RECEIVE_MODE | TRANSMIT_KEYBD );
	gbl_macro_idx = macro_num - 1;
	Set_TxRx_Labels();

	/* Enable or not, preamble xmission */
	if( isFlagClear(TRANSMIT_MODE) )
	  gtk_idle_add( Transmit_Macro, "preamble" );
	else
	  gtk_idle_add( Transmit_Macro, "nopreamble" );
  }
  else
	ClearFlag( TRANSMIT_MACRO );

} /* Select_Macro() */

/*------------------------------------------------------------------------*/

/*  Open_File()
 *
 *  Opens a file
 */

  int
Open_File( FILE **fp, char *fpath, const char *mode )
{
  struct stat buf;

  /* Check if file exixts */
  if( *fp == NULL )
  {
	/* If file does not exist, open new */
	if( stat(fpath, &buf) == -1 )
	{
	  /* Create new file */
	  if( (*fp = fopen(fpath, "w+")) == NULL )
	  {
		char mesg[108];
		snprintf( mesg, 108,
			"Failed to create\n%s\n"
			"Quit xpsk31 and correct", fpath );
		perror( fpath );
		Error_Dialog( mesg, TRUE );
		return( -1 );
	  }
	  else
	  {
		/* Re-open in requested mode */
		Close_File( fp );
		if( (*fp = fopen(fpath, mode)) == NULL )
		{
		  char mesg[108];
		  snprintf( mesg, 108,
			  "Failed to open\n%s\n"
			  "Quit xpsk31 and correct", fpath );
		  perror( fpath );
		  Error_Dialog( mesg, TRUE );
		  return( -1 );
		}
		else
		  return( FILE_NEW );
	  }
	} /* if( stat(fpath, buf) == -1 ) */
	else /* If file exists, re-open in required mode */
	{
	  if( (*fp = fopen(fpath, mode)) == NULL )
	  {
		char mesg[108];
		snprintf( mesg, 108,
			"Failed to open\n%s\n"
			"Quit xpsk31 and correct", fpath );
		perror( fpath );
		Error_Dialog( mesg, TRUE );
		return( -1 );
	  }
	  else
		return( FILE_OPEN );

	} /* else */

  } /* if( *fp == NULL ) */

  return( FILE_OPEN );

} /* Open_File() */

/*------------------------------------------------------------------------*/

/*  Close_File()
 *
 *  Closes and NULL's an open file pointer
 */

  void
Close_File( FILE **fp )
{
  if( *fp != NULL )
  {
	fclose( *fp );
	*fp = NULL;
  }
}

/*------------------------------------------------------------------------*/

/*  File_Print()
 *
 *  Prints a string to a file
 */

  gboolean
File_Print( FILE **fp, char *string )
{
  if( fprintf(*fp, "%s", string) != strlen(string) )
  {
	ClearFlag( SAVE_RECORD );
	perror( "xpsk31: fprintf" );
	Error_Dialog(
		"Error printing to file\n"
		"Quit xpsk31 and correct", TRUE );
	Close_File( fp );
	return( FALSE );
  }

  return( TRUE );

} /* File_Print() */

/*------------------------------------------------------------------------*/

/*  Locate_Line()
 *
 *  Finds a line in a text file containing a given string
 */

  gboolean
Locate_Line( char **line_idx, char *line, const char *string, FILE *fp )
{
  int err;

  /* Load lines and look for string */
  do
  {
	err = Load_Line( line, fp, "line in Log File" );
	*line_idx = strstr(line, string);
  }
  while( (err == 0) && (*line_idx == NULL) );

  /* Usually EOF is the cause of error */
  if( err )
  {
	Error_Dialog(
		"Error searching text file\n"
		"May be empty or corrupted", TRUE );
	return( FALSE );
  }

  *line_idx += strlen(string);

  return( TRUE );

} /* Locate_Line() */

/*------------------------------------------------------------------------*/

/*  Find_String()
 *
 *  Finds a string following spaces
 */

  gboolean
Locate_String( char **line_idx, char *line, const char *string )
{
  if( (*line_idx = strstr(line, string)) == NULL )
  {
	Error_Dialog(
		"Error searching station_log.txt\n"
		"May be empty or corrupted", TRUE );
	return( FALSE );
  }

  *line_idx += strlen(string);

  return( TRUE );

} /* Find_String() */

/*------------------------------------------------------------------------*/

/*  Read_QSO_Record()
 *
 *  Reads and validates QSO record entries
 */

  gboolean
Read_QSO_Record( void )
{
  GtkWidget *entry;

  /* Enter field values to QSO record structure */
  entry = lookup_widget( main_window, "callsign" );
  strncpy( qso_record.dx_call,	gtk_entry_get_text(GTK_ENTRY(entry)), 15 );
  entry = lookup_widget( main_window, "rst_in" );
  strncpy( qso_record.my_rst,	gtk_entry_get_text(GTK_ENTRY(entry)),  4 );
  entry = lookup_widget( main_window, "rst_out" );
  strncpy( qso_record.dx_rst,	gtk_entry_get_text(GTK_ENTRY(entry)),  4 );
  entry = lookup_widget( main_window, "op_name" );
  strncpy( qso_record.dx_name,	gtk_entry_get_text(GTK_ENTRY(entry)), 13 );
  entry = lookup_widget( main_window, "qth_name" );
  strncpy( qso_record.dx_qth,	gtk_entry_get_text(GTK_ENTRY(entry)), 13 );
  entry = lookup_widget( main_window, "locator" );
  strncpy( qso_record.dx_loc,	gtk_entry_get_text(GTK_ENTRY(entry)),  7 );
  entry = lookup_widget( main_window, "band" );
  strncpy( qso_record.freq,	gtk_entry_get_text(GTK_ENTRY(entry)),  7 );

  /* Enter operating mode */
  if( isFlagSet(MODE_BPSK) )
	strcpy( qso_record.mode, "BPSK" );
  else
	strcpy( qso_record.mode, "QPSK" );

  /* Validate QSO Record */
  if( (strlen(qso_record.dx_call) > 2) &&
	  (strlen(qso_record.my_rst) == 3) &&
	  (strlen(qso_record.freq)    > 0) )
	return( TRUE );
  else
	return( FALSE );

} /* Read_QSO_Record() */

/*------------------------------------------------------------------------*/

